"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
/**
 * Resolves the symbols of built in Types like Array, string...
 */
var BuiltInSymbols = (function () {
    function BuiltInSymbols(stdLibSymbols) {
        this.stdLibSymbols = stdLibSymbols;
    }
    /**
     * Creates a new instance for the given compilation
     * @param program the program to compile
     * @param compilerHost the compiler host the compiler options
     * @return {BuiltInSymbols} the built in symbols
     */
    BuiltInSymbols.create = function (program, compilerHost) {
        var symbols = BuiltInSymbols.loadStdLibVariables(program, compilerHost);
        return new BuiltInSymbols(symbols);
    };
    /**
     * Returns the symbol with the given name
     * @param name the name of the desired symbol
     * @return {undefined|ts.Symbol} the symbol with the given name if defined, undefined otherwise
     */
    BuiltInSymbols.prototype.get = function (name) {
        return this.stdLibSymbols.get(name);
    };
    BuiltInSymbols.loadStdLibVariables = function (program, compilerHost) {
        var defaultLibFileLocation = compilerHost.getDefaultLibLocation();
        var symbols = new Map();
        var stdLibFiles = program.getSourceFiles().filter(function (file) { return file.fileName.startsWith(defaultLibFileLocation); });
        var symbolKinds = ts.SymbolFlags.Variable | ts.SymbolFlags.Interface | ts.SymbolFlags.Function;
        for (var _i = 0, stdLibFiles_1 = stdLibFiles; _i < stdLibFiles_1.length; _i++) {
            var lib = stdLibFiles_1[_i];
            var stdLibVariables = program.getTypeChecker().getSymbolsInScope(lib, symbolKinds);
            for (var _a = 0, stdLibVariables_1 = stdLibVariables; _a < stdLibVariables_1.length; _a++) {
                var type = stdLibVariables_1[_a];
                symbols.set(type.name, type);
            }
        }
        // else, no stdlib included, so no array... etc
        return symbols;
    };
    return BuiltInSymbols;
}());
exports.BuiltInSymbols = BuiltInSymbols;
//# sourceMappingURL=built-in-symbols.js.map