"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var ts = require("typescript");
var util = require("util");
/**
 * Error thrown if the code generation fails. Stores the node to show the node that  caused the error
 * in the users code.
 */
var CodeGenerationDiagnostic = (function (_super) {
    __extends(CodeGenerationDiagnostic, _super);
    function CodeGenerationDiagnostic(node, code, message) {
        var _this = _super.call(this, message) || this;
        _this.node = node;
        _this.code = code;
        assert(node, "Node is not defined");
        assert(code, "code is not defined");
        assert(message, "message is not defined");
        // https://github.com/Microsoft/TypeScript-wiki/blob/master/Breaking-Changes.md#extending-built-ins-like-error-array-and-map-may-no-longer-work
        Object.setPrototypeOf(_this, CodeGenerationDiagnostic.prototype);
        return _this;
    }
    CodeGenerationDiagnostic.prototype.toDiagnostic = function () {
        return {
            code: this.code,
            messageText: this.message,
            start: this.node.getFullStart(),
            length: this.node.getFullWidth(),
            category: ts.DiagnosticCategory.Error,
            file: this.node.getSourceFile()
        };
    };
    CodeGenerationDiagnostic.builtInMethodNotSupported = function (propertyAccessExpression, objectName, methodName) {
        return CodeGenerationDiagnostic.createException(propertyAccessExpression, diagnostics.BuiltInMethodNotSupported, methodName, objectName);
    };
    CodeGenerationDiagnostic.builtInPropertyNotSupported = function (property, objectName) {
        return CodeGenerationDiagnostic.createException(property, diagnostics.BuiltInPropertyNotSupported, property.name.text, objectName);
    };
    CodeGenerationDiagnostic.builtInDoesNotSupportElementAccess = function (element, objectName) {
        return CodeGenerationDiagnostic.createException(element, diagnostics.BuiltInObjectDoesNotSupportElementAccess, objectName);
    };
    CodeGenerationDiagnostic.createException = function (node, diagnostic) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var message = util.format.apply(util, [diagnostic.message].concat(args));
        return new CodeGenerationDiagnostic(node, diagnostic.code, message);
    };
    CodeGenerationDiagnostic.unsupportedLiteralType = function (node, typeName) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedLiteralType, typeName);
    };
    CodeGenerationDiagnostic.unsupportedType = function (node, typeName) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedType, typeName);
    };
    CodeGenerationDiagnostic.unsupportedIdentifier = function (identifier) {
        return CodeGenerationDiagnostic.createException(identifier, diagnostics.UnsupportedIdentifier, identifier.text);
    };
    CodeGenerationDiagnostic.unsupportedBinaryOperation = function (binaryExpression, leftType, rightType) {
        return CodeGenerationDiagnostic.createException(binaryExpression, diagnostics.UnsupportedBinaryOperation, ts.SyntaxKind[binaryExpression.operatorToken.kind], leftType, rightType);
    };
    CodeGenerationDiagnostic.unsupportedUnaryOperation = function (node, type) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedUnaryOperation, ts.SyntaxKind[node.operator], type);
    };
    CodeGenerationDiagnostic.anonymousEntryFunctionsNotSupported = function (fun) {
        return CodeGenerationDiagnostic.createException(fun, diagnostics.AnonymousEntryFunctionsUnsupported);
    };
    CodeGenerationDiagnostic.optionalParametersInEntryFunctionNotSupported = function (optionalParameter) {
        return CodeGenerationDiagnostic.createException(optionalParameter, diagnostics.OptionalParametersNotSupportedForEntryFunction);
    };
    CodeGenerationDiagnostic.genericEntryFunctionNotSupported = function (fun) {
        return CodeGenerationDiagnostic.createException(fun, diagnostics.GenericEntryFunctionNotSuppoorted);
    };
    CodeGenerationDiagnostic.unsupportedClassReferencedBy = function (identifier) {
        return CodeGenerationDiagnostic.createException(identifier, diagnostics.UnsupportedBuiltInClass);
    };
    CodeGenerationDiagnostic.referenceToNonSpeedyJSEntryFunctionFromJS = function (identifier, speedyJSFunctionSymbol) {
        return CodeGenerationDiagnostic.createException(identifier, diagnostics.ReferenceToNonEntrySpeedyJSFunctionFromJS, speedyJSFunctionSymbol.name);
    };
    CodeGenerationDiagnostic.refereneToNonSpeedyJSFunctionFromSpeedyJS = function (identifier, nonSpeedyJsFunctionSymbol) {
        return CodeGenerationDiagnostic.createException(identifier, diagnostics.ReferenceToNonSpeedyJSFunctionFromSpeedyJS, nonSpeedyJsFunctionSymbol.name);
    };
    CodeGenerationDiagnostic.overloadedEntryFunctionNotSupported = function (fun) {
        return CodeGenerationDiagnostic.createException(fun, diagnostics.OverloadedEntryFunctionNotSupported);
    };
    CodeGenerationDiagnostic.unsupportedSyntaxKind = function (node) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedSyntaxKind, ts.SyntaxKind[node.kind]);
    };
    CodeGenerationDiagnostic.unsupportedProperty = function (propertyExpression) {
        return CodeGenerationDiagnostic.createException(propertyExpression, diagnostics.UnsupportedProperty);
    };
    CodeGenerationDiagnostic.unsupportedIndexer = function (node) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedIndexer);
    };
    CodeGenerationDiagnostic.unsupportedCast = function (node, sourceType, targetType) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedCast, sourceType, targetType);
    };
    CodeGenerationDiagnostic.implicitArrayElementCast = function (element, arrayElementType, typeOfElementRequiringImplicitCast) {
        return CodeGenerationDiagnostic.createException(element, diagnostics.UnsupportedImplicitArrayElementCast, typeOfElementRequiringImplicitCast, arrayElementType);
    };
    CodeGenerationDiagnostic.unsupportedImplicitCastOfBinaryExpressionOperands = function (binaryExpression, leftOperandType, rightOperandType) {
        return CodeGenerationDiagnostic.createException(binaryExpression, diagnostics.UnsupportedImplicitCastOfBinaryExpressionOperands, leftOperandType, rightOperandType);
    };
    CodeGenerationDiagnostic.unsupportedImplicitCastOfArgument = function (elementNotMatchingArrayElementType, parameterType, argumentType) {
        return CodeGenerationDiagnostic.createException(elementNotMatchingArrayElementType, diagnostics.UnsupportedImplicitCastOfArgument, argumentType, parameterType, parameterType);
    };
    CodeGenerationDiagnostic.unsupportedImplicitCastOfConditionalResult = function (conditionalResult, typeOfConditional, typeOfConditionResult) {
        return CodeGenerationDiagnostic.createException(conditionalResult, diagnostics.UnsupportedImplicitCastOfConditionalResult, typeOfConditionResult, typeOfConditional, typeOfConditional);
    };
    CodeGenerationDiagnostic.unsupportedElementAccessExpression = function (elementAccessExpression, argumentExpressionType) {
        return CodeGenerationDiagnostic.createException(elementAccessExpression, diagnostics.UnsupportedElementAccessExpression, argumentExpressionType || "undefined");
    };
    CodeGenerationDiagnostic.unsupportedImplicitCastOfReturnValue = function (returnStatement, declaredReturnType, returnStatementExpressionType) {
        return CodeGenerationDiagnostic.createException(returnStatement, diagnostics.UnsupportedImplicitCastOfReturnValue, returnStatementExpressionType, declaredReturnType, declaredReturnType);
    };
    CodeGenerationDiagnostic.unsupportedImplicitCast = function (node, castTargetType, actualValueType) {
        return CodeGenerationDiagnostic.createException(node, diagnostics.UnsupportedImplicitCast, actualValueType, castTargetType, castTargetType);
    };
    CodeGenerationDiagnostic.unsupportedFunctionDeclaration = function (declaration) {
        throw CodeGenerationDiagnostic.createException(declaration, diagnostics.UnsupportedFunctionDeclaration);
    };
    CodeGenerationDiagnostic.unsupportedGenericFunction = function (functionDeclaration) {
        return CodeGenerationDiagnostic.createException(functionDeclaration, diagnostics.UnsupportedGenericFunction);
    };
    CodeGenerationDiagnostic.unsupportedStaticProperties = function (propertyExpression) {
        return CodeGenerationDiagnostic.createException(propertyExpression, diagnostics.UnsupportedStaticProperty);
    };
    CodeGenerationDiagnostic.unsupportedNestedFunctionDeclaration = function (functionDeclaration) {
        return CodeGenerationDiagnostic.createException(functionDeclaration, diagnostics.UnsupportedNestedFunctionDeclaration);
    };
    CodeGenerationDiagnostic.variableDeclarationList = function (variableDeclarationList) {
        return CodeGenerationDiagnostic.createException(variableDeclarationList, diagnostics.UnsupportedVarDeclaration);
    };
    CodeGenerationDiagnostic.unsupportedOverloadedFunctionDeclaration = function (declaration) {
        return CodeGenerationDiagnostic.createException(declaration, diagnostics.UnsupportedOverloadedFunctionDeclaration);
    };
    CodeGenerationDiagnostic.unsupportedGenericClass = function (classDeclaration) {
        return CodeGenerationDiagnostic.createException(classDeclaration, diagnostics.UnsupportedGenericClass);
    };
    CodeGenerationDiagnostic.unsupportedClassInheritance = function (classDeclaration) {
        return CodeGenerationDiagnostic.createException(classDeclaration, diagnostics.UnsupportedClassInheritance);
    };
    return CodeGenerationDiagnostic;
}(Error));
exports.CodeGenerationDiagnostic = CodeGenerationDiagnostic;
/* tslint:disable:max-line-length */
var diagnostics = {
    BuiltInMethodNotSupported: {
        message: "The method '%s' of the built in object '%s' is not supported.",
        code: 100000
    },
    BuiltInPropertyNotSupported: {
        message: "The property '%s' of the built in object '%s' is not supported.",
        code: 100001
    },
    BuiltInObjectDoesNotSupportElementAccess: {
        message: "The built in object '%s' does not support element access (%s[index] or $s[index]=value).",
        code: 100002
    },
    UnsupportedBuiltInClass: {
        message: "The class referenced by this identifier is not supported.",
        code: 100003,
    },
    UnsupportedLiteralType: {
        message: "The literal type '%s' is not supported.",
        code: 100004
    },
    UnsupportedType: {
        message: "The type '%s' is not supported.",
        code: 100005
    },
    UnsupportedIdentifier: {
        message: "Unsupported type or kind of identifier '%s'.",
        code: 100006
    },
    UnsupportedBinaryOperation: {
        message: "The binary operator %s is not supported for the left and right hand side types '%s' and '%s'.",
        code: 100007
    },
    UnsupportedUnaryOperation: {
        message: "The unary operator %s is not supported for the type '%s'.",
        code: 100008
    },
    AnonymousEntryFunctionsUnsupported: {
        message: "SpeedyJS entry functions need to have a name.",
        code: 100009
    },
    ReferenceToNonEntrySpeedyJSFunctionFromJS: {
        message: "SpeedyJS functions referenced from 'normal' JavaScript code needs to be async (the async modifier is missing on the declaration of '%s').",
        code: 100010
    },
    OptionalParametersNotSupportedForEntryFunction: {
        message: "Optional parameters or variadic parameters are not supported for SpeedyJS entry functions.",
        code: 100011
    },
    GenericEntryFunctionNotSuppoorted: {
        message: "Generic SpeedyJS entry functions are not supported.",
        code: 100012
    },
    OverloadedEntryFunctionNotSupported: {
        message: "SpeedyJS entry function cannot be overloaded.",
        code: 100013
    },
    UnsupportedSyntaxKind: {
        message: "The syntax kind '%s' is not yet supported.",
        code: 100014
    },
    UnsupportedProperty: {
        message: "The kind of property is not yet supported, only object properties are supported.",
        code: 1000015
    },
    UnsupportedIndexer: {
        message: "The kind of indexer is not yet supported",
        code: 1000016
    },
    UnsupportedCast: {
        message: "Casting from '%s' to '%s' is not yet supported",
        code: 1000017
    },
    UnsupportedImplicitArrayElementCast: {
        message: "The array element of type '%s' requires an implicit cast to the array element type '%s'. Implicit casts are not supported. An explicit cast of the element to the array element type is required.",
        code: 1000018
    },
    UnsupportedImplicitCastOfBinaryExpressionOperands: {
        message: "No implicit cast for the binary expressions operands (left: %s, right: %s) exists. An explicit cast of either of the operands to the other's type is required.",
        code: 1000019
    },
    UnsupportedImplicitCastOfArgument: {
        message: "Unsupported implicit cast of the argument with the type '%s' to the expected parameter type '%s'. An explicit cast of the argument to the type '%s' is required.",
        code: 1000020
    },
    UnsupportedImplicitCastOfConditionalResult: {
        message: "Unsupported implicit cast of conditional case with type '%s' to the type '%s' of the whole conditional expression. An explicit cast of the conditional case to the type '%s' is required.",
        code: 1000021
    },
    UnsupportedElementAccessExpression: {
        message: "Unsupported element access expression with indexer of type '%s'. Only element access expressions with an integer index are supported.",
        code: 1000022
    },
    UnsupportedImplicitCastOfReturnValue: {
        message: "Unsupported implicit cast of return value with the type '%s' to function's return type '%s'. An explicit cast of the return value to the expected return type '%s' is required.",
        code: 1000023
    },
    UnsupportedImplicitCast: {
        message: "Unsupported implicit cast of value with the type '%s' to the expected type '%s'. An explicit cast of the value to the type '%s' is required.",
        code: 1000024
    },
    UnsupportedGenericFunction: {
        message: "Generic functions and methods are not yet supported.",
        code: 1000025
    },
    UnsupportedStaticProperty: {
        message: "Static methods and properties are not yet supported.",
        code: 1000026
    },
    UnsupportedNestedFunctionDeclaration: {
        message: "Functions nested inside of other functions that potentially make use of a closure are not yet supported",
        code: 1000027
    },
    UnsupportedFunctionDeclaration: {
        message: "Unsupported function declaration. Only function and method declarations are supported.",
        code: 1000028
    },
    UnsupportedVarDeclaration: {
        message: "Unsupported 'var' declaration of variable. Only variables with block scope ('let' and 'const') are supported.",
        code: 1000029
    },
    UnsupportedOverloadedFunctionDeclaration: {
        message: "Overloaded functions are not yet supported.",
        code: 1000030
    },
    UnsupportedGenericClass: {
        message: "Generic classes are not yet supported.",
        code: 1000031
    },
    UnsupportedClassInheritance: {
        message: "Class inheritance is not yet supported.",
        code: 1000032
    },
    ReferenceToNonSpeedyJSFunctionFromSpeedyJS: {
        message: "The Speedy.js function cannot reference the regular JavaScript function '%s'. Calling JavaScript functions from Speedy.js is not yet supported. Either remove the function call or make the called function a Speedy.js function by adding the \"use speedyjs\" directive.",
        code: 1000033
    }
};
//# sourceMappingURL=code-generation-diagnostic.js.map