"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var types_1 = require("../util/types");
var array_class_reference_1 = require("../value/array-class-reference");
/**
 * Code Generator for [1, 2, ...] array expressions
 */
var ArrayLiteralExpressionCodeGenerator = (function () {
    function ArrayLiteralExpressionCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.ArrayLiteralExpression;
    }
    ArrayLiteralExpressionCodeGenerator.prototype.generate = function (arrayLiteral, context) {
        var type = context.typeChecker.getTypeAtLocation(arrayLiteral);
        var elementType = types_1.getArrayElementType(type);
        var elements = new Array(arrayLiteral.elements.length);
        for (var i = 0; i < elements.length; ++i) {
            var element = arrayLiteral.elements[i];
            var casted = context.generateValue(element).castImplicit(elementType, context);
            if (!casted) {
                var arrayElementTypeName = context.typeChecker.typeToString(elementType);
                var elementTypeName = context.typeChecker.typeToString(context.typeChecker.getTypeAtLocation(element));
                throw code_generation_diagnostic_1.CodeGenerationDiagnostic.implicitArrayElementCast(element, arrayElementTypeName, elementTypeName);
            }
            elements[i] = casted.generateIR(context);
        }
        return array_class_reference_1.ArrayClassReference.fromLiteral(type, elements, context);
    };
    return ArrayLiteralExpressionCodeGenerator;
}());
exports.default = ArrayLiteralExpressionCodeGenerator;
//# sourceMappingURL=array-literal-expression-code-generator.js.map