"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var primitive_1 = require("../value/primitive");
var AsExpressionCodeGenerator = (function () {
    function AsExpressionCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.AsExpression;
    }
    AsExpressionCodeGenerator.prototype.generate = function (node, context) {
        var value = context.generateValue(node.expression);
        var sourceType = context.typeChecker.getTypeAtLocation(node.expression);
        var targetType = context.typeChecker.getTypeAtLocation(node);
        var castedValue;
        if (targetType.flags & ts.TypeFlags.BooleanLike) {
            castedValue = new primitive_1.Primitive(primitive_1.Primitive.toBoolean(value, sourceType, context), targetType);
        }
        else if (targetType.flags & ts.TypeFlags.IntLike) {
            castedValue = primitive_1.Primitive.toInt32(value, sourceType, targetType, context);
        }
        else if (targetType.flags & ts.TypeFlags.NumberLike) {
            castedValue = primitive_1.Primitive.toNumber(value, sourceType, targetType, context);
        }
        else {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedCast(node, context.typeChecker.typeToString(sourceType), context.typeChecker.typeToString(targetType));
        }
        return castedValue;
    };
    return AsExpressionCodeGenerator;
}());
exports.default = AsExpressionCodeGenerator;
//# sourceMappingURL=as-expression-code-generator.js.map