"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var primitive_1 = require("../value/primitive");
/**
 * Code Generator for condition expressions (condition ? whenTrue : whenFalse);
 */
var ConditionalExpressionCodeGenerator = (function () {
    function ConditionalExpressionCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.ConditionalExpression;
    }
    ConditionalExpressionCodeGenerator.prototype.generate = function (node, context) {
        var whenTrueType = context.typeChecker.getTypeAtLocation(node.whenTrue);
        var whenFalseType = context.typeChecker.getTypeAtLocation(node.whenFalse);
        var conditionalType = context.typeChecker.getTypeAtLocation(node);
        var condition = context.generateValue(node.condition);
        var conditionBool = primitive_1.Primitive.toBoolean(condition.generateIR(context), context.typeChecker.getTypeAtLocation(node.condition), context);
        var whenTrue = context.generateValue(node.whenTrue).castImplicit(conditionalType, context);
        var whenFalse = context.generateValue(node.whenFalse).castImplicit(conditionalType, context);
        var conditionalTypeName = context.typeChecker.typeToString(conditionalType);
        if (!whenFalse) {
            var whenFalseTypeName = context.typeChecker.typeToString(whenFalseType);
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedImplicitCastOfConditionalResult(node.whenFalse, conditionalTypeName, whenFalseTypeName);
        }
        if (!whenTrue) {
            var whenTrueTypeName = context.typeChecker.typeToString(whenTrueType);
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedImplicitCastOfConditionalResult(node.whenTrue, conditionalTypeName, whenTrueTypeName);
        }
        var result = context.builder.createSelect(conditionBool, whenTrue.generateIR(context), whenFalse.generateIR(context), "cond");
        return context.value(result, context.typeChecker.getTypeAtLocation(node));
    };
    return ConditionalExpressionCodeGenerator;
}());
exports.default = ConditionalExpressionCodeGenerator;
//# sourceMappingURL=conditional-expression-code-generator.js.map