"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
/**
 * Element Access Code Generator, eg. array[index]
 */
var ElementAccessExpressionCodeGenerator = (function () {
    function ElementAccessExpressionCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.ElementAccessExpression;
    }
    ElementAccessExpressionCodeGenerator.prototype.generate = function (node, context) {
        if (!node.argumentExpression) {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedElementAccessExpression(node);
        }
        var argumentExpressionType = context.typeChecker.getTypeAtLocation(node.argumentExpression);
        if (!(argumentExpressionType.flags & ts.TypeFlags.IntLike)) {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedElementAccessExpression(node, context.typeChecker.typeToString(argumentExpressionType));
        }
        var value = context.generateValue(node.expression).dereference(context);
        if (value.isObject()) {
            return value.getIndexer(node, context);
        }
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedIndexer(node);
    };
    return ElementAccessExpressionCodeGenerator;
}());
exports.default = ElementAccessExpressionCodeGenerator;
//# sourceMappingURL=element-access-expression-code-generator.js.map