"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var primitive_1 = require("../value/primitive");
var FirstLiteralTokenCodeGenerator = (function () {
    function FirstLiteralTokenCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.FirstLiteralToken;
    }
    FirstLiteralTokenCodeGenerator.prototype.generate = function (node, context) {
        var type = context.typeChecker.getTypeAtLocation(node);
        var value;
        if (type.flags & ts.TypeFlags.IntLike) {
            value = llvm.ConstantInt.get(context.llvmContext, +node.text);
        }
        else if (type.flags & ts.TypeFlags.NumberLike) {
            value = llvm.ConstantFP.get(context.llvmContext, +node.text);
        }
        else {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedLiteralType(node, context.typeChecker.typeToString(type));
        }
        return new primitive_1.Primitive(value, type);
    };
    return FirstLiteralTokenCodeGenerator;
}());
exports.default = FirstLiteralTokenCodeGenerator;
//# sourceMappingURL=first-literal-token-code-generator.js.map