"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var conditions_1 = require("../util/conditions");
var ForStatementCodeGenerator = (function () {
    function ForStatementCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.ForStatement;
    }
    ForStatementCodeGenerator.prototype.generate = function (forStatement, context) {
        if (forStatement.initializer) {
            context.generate(forStatement.initializer);
        }
        var head;
        var fun = context.scope.enclosingFunction;
        var end = llvm.BasicBlock.create(context.llvmContext, "for.end");
        var body = llvm.BasicBlock.create(context.llvmContext, "for.body");
        var incrementer = llvm.BasicBlock.create(context.llvmContext, "for.inc");
        if (forStatement.condition) {
            var forBlock = llvm.BasicBlock.create(context.llvmContext, "for.cond", fun);
            context.builder.createBr(forBlock);
            context.builder.setInsertionPoint(forBlock);
            conditions_1.generateCondition(forStatement.condition, body, end, context);
            head = forBlock;
        }
        else {
            context.builder.createBr(body);
            head = body;
        }
        this.setContinueAndBreakLabels(forStatement.parent, context, incrementer, end);
        fun.addBasicBlock(body);
        context.builder.setInsertionPoint(body);
        context.generate(forStatement.statement);
        body = context.builder.getInsertBlock();
        if (!body.getTerminator()) {
            context.builder.createBr(incrementer);
        }
        fun.addBasicBlock(incrementer);
        context.builder.setInsertionPoint(incrementer);
        if (forStatement.incrementor) {
            context.generate(forStatement.incrementor);
        }
        context.builder.createBr(head);
        fun.addBasicBlock(end);
        context.builder.setInsertionPoint(end);
    };
    ForStatementCodeGenerator.prototype.setContinueAndBreakLabels = function (parent, context, incrementer, end) {
        context.scope.setContinueBlock(incrementer);
        context.scope.setBreakBlock(end);
        if (parent && parent.kind === ts.SyntaxKind.LabeledStatement) {
            var labeledStatement = parent;
            if (labeledStatement.label) {
                context.scope.setContinueBlock(incrementer, labeledStatement.label.text);
                context.scope.setBreakBlock(end, labeledStatement.label.text);
            }
        }
    };
    return ForStatementCodeGenerator;
}());
exports.default = ForStatementCodeGenerator;
//# sourceMappingURL=for-statement-code-generator.js.map