"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var undefined_1 = require("../value/undefined");
var unresolved_function_reference_1 = require("../value/unresolved-function-reference");
var IdentifierCodeGenerator = (function () {
    function IdentifierCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.Identifier;
    }
    IdentifierCodeGenerator.prototype.generate = function (identifier, context) {
        var symbol = context.typeChecker.getSymbolAtLocation(identifier);
        if (context.typeChecker.isUndefinedSymbol(symbol)) {
            return undefined_1.Undefined.create(context);
        }
        if (symbol.flags & ts.SymbolFlags.Function) {
            return IdentifierCodeGenerator.getFunction(symbol, identifier, context);
        }
        if (symbol.flags & ts.SymbolFlags.Variable && context.scope.hasVariable(symbol)) {
            return context.scope.getVariable(symbol);
        }
        if (symbol.flags & ts.SymbolFlags.Type) {
            var classType = context.typeChecker.getDeclaredTypeOfSymbol(symbol);
            var classReference = context.resolveClass(classType, symbol);
            if (classReference) {
                return classReference;
            }
        }
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedIdentifier(identifier);
    };
    IdentifierCodeGenerator.getFunction = function (symbol, identifier, context) {
        if (context.scope.hasFunction(symbol)) {
            return context.scope.getFunction(symbol);
        }
        var type = context.typeChecker.getTypeAtLocation(identifier);
        var apparentType = context.typeChecker.getApparentType(type);
        var callSignatures = context.typeChecker.getSignaturesOfType(apparentType, ts.SignatureKind.Call);
        return unresolved_function_reference_1.UnresolvedFunctionReference.createFunction(callSignatures, context);
    };
    return IdentifierCodeGenerator;
}());
exports.default = IdentifierCodeGenerator;
//# sourceMappingURL=identifier-code-generator.js.map