"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var conditions_1 = require("../util/conditions");
var IfStatementCodeGenerator = (function () {
    function IfStatementCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.IfStatement;
    }
    IfStatementCodeGenerator.prototype.generate = function (ifStatement, context) {
        var fun = context.scope.enclosingFunction;
        var thenBlock = llvm.BasicBlock.create(context.llvmContext, "if.then");
        var elseBlock = llvm.BasicBlock.create(context.llvmContext, "if.else");
        var end = llvm.BasicBlock.create(context.llvmContext, "if.end");
        conditions_1.generateCondition(ifStatement.expression, thenBlock, elseBlock, context);
        fun.addBasicBlock(thenBlock);
        context.builder.setInsertionPoint(thenBlock);
        context.generate(ifStatement.thenStatement);
        thenBlock = context.builder.getInsertBlock();
        if (!thenBlock.getTerminator()) {
            context.builder.createBr(end);
        }
        fun.addBasicBlock(elseBlock);
        context.builder.setInsertionPoint(elseBlock);
        if (ifStatement.elseStatement) {
            context.generate(ifStatement.elseStatement);
            elseBlock = context.builder.getInsertBlock();
        }
        if (elseBlock.empty) {
            elseBlock.replaceAllUsesWith(end);
            elseBlock.eraseFromParent();
        }
        else if (!elseBlock.getTerminator()) {
            context.builder.createBr(end);
        }
        fun.addBasicBlock(end);
        context.builder.setInsertionPoint(end);
    };
    return IfStatementCodeGenerator;
}());
exports.default = IfStatementCodeGenerator;
//# sourceMappingURL=if-statement-code-generator.js.map