"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
/**
 * Code generator for labeled statements, e.g.
 * for (let i=0; i < 4; ++i) {
 *       checkj: for (let j = 8; j > 4; --j) {
 *           j -= 1;
 *
 *           if ((j % 2) === 0) {
 *               continue checkj;
 *           }
 *       }
 *   }
 */
var LabeledStatementCodeGenerator = (function () {
    function LabeledStatementCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.LabeledStatement;
    }
    LabeledStatementCodeGenerator.prototype.generate = function (node, context) {
        var end = llvm.BasicBlock.create(context.llvmContext, node.label.text + ".end");
        context.scope.setBreakBlock(end, node.label.text);
        context.generate(node.statement);
        // The block might not be used, or a for loop overrides the label target with the for.end
        if (!end.useEmpty()) {
            if (!context.builder.getInsertBlock().getTerminator()) {
                context.builder.createBr(end);
            }
            context.scope.enclosingFunction.addBasicBlock(end);
            context.builder.setInsertionPoint(end);
        }
        else {
            end.release();
        }
    };
    return LabeledStatementCodeGenerator;
}());
exports.default = LabeledStatementCodeGenerator;
//# sourceMappingURL=labeled-statement-code-generator.js.map