"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var PostfixUnaryExpressionCodeGenerator = (function () {
    function PostfixUnaryExpressionCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.PostfixUnaryExpression;
    }
    PostfixUnaryExpressionCodeGenerator.prototype.generate = function (postfixUnaryExpression, context) {
        var left = context.generateValue(postfixUnaryExpression.operand);
        var before = left.generateIR(context);
        var updated;
        var operandType = context.typeChecker.getTypeAtLocation(postfixUnaryExpression.operand);
        switch (postfixUnaryExpression.operator) {
            case ts.SyntaxKind.PlusPlusToken:
                if (operandType.flags & ts.TypeFlags.IntLike) {
                    updated = context.builder.createAdd(before, llvm.ConstantInt.get(context.llvmContext, 1), "inc");
                }
                else if (operandType.flags & ts.TypeFlags.NumberLike) {
                    updated = context.builder.createFAdd(before, llvm.ConstantFP.get(context.llvmContext, 1), "inc");
                }
                break;
            case ts.SyntaxKind.MinusMinusToken:
                if (operandType.flags & ts.TypeFlags.IntLike) {
                    updated = context.builder.createAdd(before, llvm.ConstantInt.get(context.llvmContext, -1), "dec");
                }
                else if (operandType.flags & ts.TypeFlags.NumberLike) {
                    updated = context.builder.createFAdd(before, llvm.ConstantFP.get(context.llvmContext, -1), "dec");
                }
                break;
        }
        if (!updated) {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedUnaryOperation(postfixUnaryExpression, context.typeChecker.typeToString(operandType));
        }
        context.assignValue(left, context.value(updated, operandType));
        return context.value(before, operandType);
    };
    return PostfixUnaryExpressionCodeGenerator;
}());
exports.default = PostfixUnaryExpressionCodeGenerator;
//# sourceMappingURL=postfix-unary-expression-code-generator.js.map