"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var ReturnStatementCodeGenerator = (function () {
    function ReturnStatementCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.ReturnStatement;
    }
    ReturnStatementCodeGenerator.prototype.generate = function (returnStatement, context) {
        var returnAllocation = context.scope.returnAllocation;
        var returnBlock = context.scope.returnBlock;
        if (returnStatement.expression) {
            assert(returnAllocation, "No return allocation present but return statement is");
            var returnType = getExpectedReturnType(returnStatement, context.typeChecker);
            var expressionType = context.typeChecker.getTypeAtLocation(returnStatement.expression);
            var returnValue = context.generateValue(returnStatement.expression);
            var casted = returnValue.castImplicit(returnType, context);
            if (!casted) {
                throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedImplicitCastOfReturnValue(returnStatement, context.typeChecker.typeToString(returnType), context.typeChecker.typeToString(expressionType));
            }
            returnAllocation.generateAssignmentIR(casted, context);
        }
        assert(returnBlock, "No return block present (not inside of a function?)");
        context.builder.createBr(returnBlock);
    };
    return ReturnStatementCodeGenerator;
}());
function getExpectedReturnType(node, typeChecker) {
    function findEnclosingFunction(current) {
        // tslint:disable-next-line:max-line-length
        if (current.kind === ts.SyntaxKind.FunctionExpression || current.kind === ts.SyntaxKind.FunctionDeclaration || current.kind === ts.SyntaxKind.ArrowFunction || current.kind === ts.SyntaxKind.MethodDeclaration) {
            return current;
        }
        assert(current.parent, "Node is not inside of a function");
        return findEnclosingFunction(current.parent);
    }
    var fun = findEnclosingFunction(node);
    var signature = typeChecker.getSignatureFromDeclaration(fun);
    return signature.getReturnType();
}
exports.default = ReturnStatementCodeGenerator;
//# sourceMappingURL=return-statement-code-generator.js.map