"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
/**
 * Code generators for string literals "hello world"
 */
var StringLiteralCodeGenerator = (function () {
    function StringLiteralCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.StringLiteral;
    }
    StringLiteralCodeGenerator.prototype.generate = function (node, context) {
        // e.g. "use speedyjs";
        if (StringLiteralCodeGenerator.isPrologueDirective(node)) {
            return;
        }
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedSyntaxKind(node);
    };
    /**
     * Test if this string literal is a prologue (e.g. "use strict")
     * @param node the string literal to test for
     * @return {boolean} true if it is a prologue, false otherwise
     */
    StringLiteralCodeGenerator.isPrologueDirective = function (node) {
        return !!node.parent && node.parent.kind === ts.SyntaxKind.ExpressionStatement;
    };
    return StringLiteralCodeGenerator;
}());
exports.default = StringLiteralCodeGenerator;
//# sourceMappingURL=string-literal-code-generator.js.map