"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var allocation_1 = require("../value/allocation");
var VariableDeclarationCodeGenerator = (function () {
    function VariableDeclarationCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.VariableDeclaration;
    }
    VariableDeclarationCodeGenerator.prototype.generate = function (variableDeclaration, context) {
        var symbol = context.typeChecker.getSymbolAtLocation(variableDeclaration.name);
        var type = context.typeChecker.getTypeAtLocation(variableDeclaration);
        var allocation = allocation_1.Allocation.create(type, context, symbol.name);
        if (variableDeclaration.initializer) {
            var initializerType = context.typeChecker.getTypeAtLocation(variableDeclaration.initializer);
            var initializer = context.generateValue(variableDeclaration.initializer);
            var castedInitializer = initializer.castImplicit(type, context);
            if (!castedInitializer) {
                throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedImplicitCast(variableDeclaration, context.typeChecker.typeToString(type), context.typeChecker.typeToString(initializerType));
            }
            allocation.generateAssignmentIR(castedInitializer, context);
        }
        // otherwise no initialization is needed. Typescript complains if a variable is accessed before it has been assigned a value
        // Furthermore, the WebAssembly standard guarantees that local variables are default initialized.
        context.scope.addVariable(symbol, allocation);
    };
    return VariableDeclarationCodeGenerator;
}());
exports.default = VariableDeclarationCodeGenerator;
//# sourceMappingURL=variable-declaration-code-generator.js.map