"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var conditions_1 = require("../util/conditions");
var WhileStatementCodeGenerator = (function () {
    function WhileStatementCodeGenerator() {
        this.syntaxKind = ts.SyntaxKind.WhileStatement;
    }
    WhileStatementCodeGenerator.prototype.generate = function (whileStatement, context) {
        var condition = llvm.BasicBlock.create(context.llvmContext, "while.cond", context.scope.enclosingFunction);
        var body = llvm.BasicBlock.create(context.llvmContext, "while.body");
        var end = llvm.BasicBlock.create(context.llvmContext, "while.end");
        context.scope.setContinueBlock(condition);
        context.scope.setBreakBlock(end);
        this.setContinueAndBreakLabels(whileStatement.parent, context, condition, end);
        context.builder.createBr(condition);
        context.builder.setInsertionPoint(condition);
        conditions_1.generateCondition(whileStatement.expression, body, end, context);
        context.scope.enclosingFunction.addBasicBlock(body);
        context.builder.setInsertionPoint(body);
        context.generate(whileStatement.statement);
        body = context.builder.getInsertBlock();
        if (!body.getTerminator()) {
            context.builder.createBr(condition);
        }
        context.scope.enclosingFunction.addBasicBlock(end);
        context.builder.setInsertionPoint(end);
    };
    WhileStatementCodeGenerator.prototype.setContinueAndBreakLabels = function (parent, context, condition, end) {
        context.scope.setContinueBlock(condition);
        context.scope.setBreakBlock(end);
        if (parent && parent.kind === ts.SyntaxKind.LabeledStatement) {
            var labeledStatement = parent;
            if (labeledStatement.label) {
                context.scope.setContinueBlock(condition, labeledStatement.label.text);
                context.scope.setBreakBlock(end, labeledStatement.label.text);
            }
        }
    };
    return WhileStatementCodeGenerator;
}());
exports.default = WhileStatementCodeGenerator;
//# sourceMappingURL=while-statement-code-generator.js.map