"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var default_name_mangler_1 = require("../default-name-mangler");
var runtime_system_name_mangler_1 = require("../runtime-system-name-mangler");
var object_index_reference_1 = require("../value/object-index-reference");
var types_1 = require("./types");
var ObjectIndexReferenceBuilder = (function () {
    function ObjectIndexReferenceBuilder(element, context) {
        this.element = element;
        this.context = context;
        this.runtimeFn = false;
    }
    ObjectIndexReferenceBuilder.forElement = function (element, context) {
        return new ObjectIndexReferenceBuilder(element, context);
    };
    Object.defineProperty(ObjectIndexReferenceBuilder.prototype, "indexType", {
        get: function () {
            return llvm.Type.getInt32Ty(this.context.llvmContext);
        },
        enumerable: true,
        configurable: true
    });
    ObjectIndexReferenceBuilder.prototype.fromRuntime = function () {
        this.runtimeFn = true;
        return this;
    };
    ObjectIndexReferenceBuilder.prototype.build = function (objectReference) {
        var elementType = this.context.typeChecker.getTypeAtLocation(this.element);
        var llvmElementType = types_1.toLLVMType(elementType, this.context);
        var llvmThisType = types_1.toLLVMType(objectReference.type, this.context);
        var getter = this.createGetter(llvmThisType, llvmElementType, objectReference);
        var setter = this.createSetter(llvmThisType, llvmElementType, objectReference);
        var index = this.context.generateValue(this.element.argumentExpression);
        return new object_index_reference_1.ObjectIndexReference(elementType, objectReference, index, getter, setter);
    };
    ObjectIndexReferenceBuilder.prototype.getNameMangler = function () {
        return this.runtimeFn ? new runtime_system_name_mangler_1.RuntimeSystemNameMangler(this.context.compilationContext) : new default_name_mangler_1.DefaultNameMangler(this.context.compilationContext);
    };
    ObjectIndexReferenceBuilder.prototype.createGetter = function (thisType, elementType, objectReference) {
        var getterName = this.getNameMangler().mangleIndexer(this.element, false);
        var getter = this.context.module.getFunction(getterName);
        if (!getter) {
            var getterType = llvm.FunctionType.get(elementType, [thisType, this.indexType], false);
            getter = llvm.Function.create(getterType, llvm.LinkageTypes.ExternalLinkage, getterName, this.context.module);
            getter.addFnAttr(llvm.Attribute.AttrKind.ReadOnly);
            getter.addFnAttr(llvm.Attribute.AttrKind.AlwaysInline);
            getter.addFnAttr(llvm.Attribute.AttrKind.NoUnwind);
            getter.addFnAttr(llvm.Attribute.AttrKind.NoRecurse);
            var self = getter.getArguments()[0];
            self.addDereferenceableAttr(objectReference.getTypeStoreSize(this.context));
            self.addAttr(llvm.Attribute.AttrKind.NoCapture);
        }
        return getter;
    };
    ObjectIndexReferenceBuilder.prototype.createSetter = function (thisType, elementType, objectReference) {
        var setterName = this.getNameMangler().mangleIndexer(this.element, true);
        var setter = this.context.module.getFunction(setterName);
        if (!setter) {
            var setterType = llvm.FunctionType.get(llvm.Type.getVoidTy(this.context.llvmContext), [thisType, this.indexType, elementType], false);
            setter = llvm.Function.create(setterType, llvm.LinkageTypes.ExternalLinkage, setterName, this.context.module);
            setter.addFnAttr(llvm.Attribute.AttrKind.AlwaysInline);
            var self = setter.getArguments()[0];
            self.addDereferenceableAttr(objectReference.getTypeStoreSize(this.context));
            self.addAttr(llvm.Attribute.AttrKind.NoCapture);
        }
        return setter;
    };
    return ObjectIndexReferenceBuilder;
}());
exports.ObjectIndexReferenceBuilder = ObjectIndexReferenceBuilder;
//# sourceMappingURL=object-index-reference-builder.js.map