"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var runtime_system_name_mangler_1 = require("../runtime-system-name-mangler");
var types_1 = require("../util/types");
var array_reference_1 = require("./array-reference");
var class_reference_1 = require("./class-reference");
var resolved_function_1 = require("./resolved-function");
var resolved_function_reference_1 = require("./resolved-function-reference");
var unresolved_function_reference_1 = require("./unresolved-function-reference");
/**
 * Implements the static methods of the Array<T> class
 */
var ArrayClassReference = (function (_super) {
    __extends(ArrayClassReference, _super);
    function ArrayClassReference(typeInformation, symbol, compilationContext) {
        var _this = _super.call(this, typeInformation, symbol, compilationContext) || this;
        _this.llvmTypes = new Map();
        return _this;
    }
    ArrayClassReference.create = function (symbol, context) {
        var typeInformation = class_reference_1.ClassReference.createTypeDescriptor(symbol, context);
        return new ArrayClassReference(typeInformation, symbol, context.compilationContext);
    };
    /**
     * Creates an array instance from an array literal
     * @param type the type of the array
     * @param elements the elements that are to be stored in the created array
     * @param context the code generation context
     * @return {ArrayReference} the created array
     */
    ArrayClassReference.fromLiteral = function (type, elements, context) {
        var parameters = [resolved_function_1.createResolvedParameter("items", type, false, undefined, true)];
        var resolvedConstructor = resolved_function_1.createResolvedFunction("constructor", [], parameters, type, type.getSymbol().getDeclarations()[0].getSourceFile(), type);
        var constructorFunction = resolved_function_reference_1.ResolvedFunctionReference.createRuntimeFunction(resolvedConstructor, context);
        context.requiresGc = true;
        return constructorFunction.invokeWith(elements, context);
    };
    ArrayClassReference.fromCArray = function (type, elementsPtr, length, context) {
        var constructorName = new runtime_system_name_mangler_1.RuntimeSystemNameMangler(context.compilationContext).mangleMethodName(type, "constructor", [type]);
        var constructorFn = context.module.getFunction(constructorName);
        if (!constructorFn) {
            var elementType = types_1.toLLVMType(types_1.getArrayElementType(type), context);
            var constructorType = llvm.FunctionType.get(types_1.toLLVMType(type, context), [
                llvm.PointerType.get(elementType, 0),
                llvm.Type.getInt32Ty(context.llvmContext)
            ], false);
            constructorFn = llvm.Function.create(constructorType, llvm.LinkageTypes.ExternalLinkage, constructorName, context.module);
            constructorFn.addFnAttr(llvm.Attribute.AttrKind.AlwaysInline);
        }
        var arrayPtr = context.builder.createCall(constructorFn, [elementsPtr, length]);
        context.requiresGc = true;
        return context.value(arrayPtr, type);
    };
    ArrayClassReference.prototype.objectFor = function (address, type) {
        return new array_reference_1.ArrayReference(address, type, this);
    };
    ArrayClassReference.prototype.getFields = function () {
        return [];
    };
    ArrayClassReference.prototype.getConstructor = function (newExpression, context) {
        var constructorSignature = context.typeChecker.getResolvedSignature(newExpression);
        context.requiresGc = true;
        return unresolved_function_reference_1.UnresolvedFunctionReference.createRuntimeFunction([constructorSignature], context);
    };
    ArrayClassReference.prototype.getLLVMType = function (type, context) {
        var elementType = types_1.getArrayElementType(type);
        if (types_1.isMaybeObjectType(elementType)) {
            elementType = elementType.getNonNullableType();
        }
        var existing = this.llvmTypes.get(elementType);
        if (existing) {
            return existing;
        }
        var forwardDeclaration = llvm.StructType.create(context.llvmContext, "class.Array");
        this.llvmTypes.set(elementType, forwardDeclaration);
        var llvmElementType;
        if (elementType.flags & ts.TypeFlags.Object) {
            llvmElementType = llvm.Type.getInt8PtrTy(context.llvmContext);
        }
        else {
            llvmElementType = types_1.toLLVMType(elementType, context);
        }
        forwardDeclaration.setBody([
            llvmElementType.getPointerTo(),
            llvm.Type.getInt32Ty(context.llvmContext),
            llvm.Type.getInt32Ty(context.llvmContext)
        ]);
        return forwardDeclaration;
    };
    return ArrayClassReference;
}(class_reference_1.ClassReference));
exports.ArrayClassReference = ArrayClassReference;
//# sourceMappingURL=array-class-reference.js.map