"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var computed_object_property_reference_builder_1 = require("../util/computed-object-property-reference-builder");
var object_index_reference_builder_1 = require("../util/object-index-reference-builder");
var types_1 = require("../util/types");
var built_in_object_reference_1 = require("./built-in-object-reference");
var unresolved_method_reference_1 = require("./unresolved-method-reference");
/**
 * Reference to an Array<T> Object
 */
var ArrayReference = (function (_super) {
    __extends(ArrayReference, _super);
    /**
     * Creates a new instance
     * @param address the address of the array object
     * @param arrayType the type of the array
     * @param arrayClass the arrayClass
     */
    function ArrayReference(address, arrayType, arrayClass) {
        var _this = _super.call(this, address, arrayType, arrayClass) || this;
        _this.elementType = types_1.getArrayElementType(arrayType);
        return _this;
    }
    Object.defineProperty(ArrayReference.prototype, "typeName", {
        get: function () {
            return "Array";
        },
        enumerable: true,
        configurable: true
    });
    ArrayReference.prototype.createFunctionFor = function (symbol, signatures, propertyAccess, context) {
        switch (symbol.name) {
            case "fill":
            case "unshift":
            case "push":
            case "pop":
            case "shift":
            case "slice":
            case "splice":
                return unresolved_method_reference_1.UnresolvedMethodReference.createRuntimeMethod(this, signatures, context);
            default:
                return this.throwUnsupportedBuiltIn(propertyAccess);
        }
    };
    ArrayReference.prototype.createPropertyReference = function (symbol, propertyAccess, context) {
        switch (symbol.name) {
            case "length":
                return computed_object_property_reference_builder_1.ComputedObjectPropertyReferenceBuilder
                    .forProperty(propertyAccess, context)
                    .fromRuntime()
                    .build(this);
            default:
                return this.throwUnsupportedBuiltIn(propertyAccess);
        }
    };
    ArrayReference.prototype.getIndexer = function (elementAccessExpression, context) {
        return object_index_reference_builder_1.ObjectIndexReferenceBuilder
            .forElement(elementAccessExpression, context)
            .fromRuntime()
            .build(this);
    };
    return ArrayReference;
}(built_in_object_reference_1.BuiltInObjectReference));
exports.ArrayReference = ArrayReference;
//# sourceMappingURL=array-reference.js.map