"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var allocation_1 = require("./allocation");
var class_reference_1 = require("./class-reference");
var math_object_reference_1 = require("./math-object-reference");
var MathClassReference = (function (_super) {
    __extends(MathClassReference, _super);
    function MathClassReference(typeInformation, symbol, compilationContext) {
        return _super.call(this, typeInformation, symbol, compilationContext) || this;
    }
    MathClassReference.create = function (symbol, context) {
        var typeInformation = class_reference_1.ClassReference.createTypeDescriptor(symbol, context);
        return new MathClassReference(typeInformation, symbol, context.compilationContext);
    };
    /**
     * Creates the global Math object
     * @param symbol the symbol of the math object
     * @param context the code generation context
     * @return {Allocation} the allocation of the global math object
     */
    MathClassReference.prototype.createGlobalVariable = function (symbol, context) {
        var mathType = context.typeChecker.getDeclaredTypeOfSymbol(symbol);
        var structType = this.getObjectType(mathType, context);
        var struct = llvm.ConstantStruct.get(structType, [llvm.ConstantInt.get(context.llvmContext, 0, 1)]);
        var storage = new llvm.GlobalVariable(context.module, structType, true, llvm.LinkageTypes.PrivateLinkage, struct, "Math_object");
        var ptr = new llvm.GlobalVariable(context.module, structType.getPointerTo(), true, llvm.LinkageTypes.PrivateLinkage, storage, "Math_ptr");
        return allocation_1.Allocation.createGlobal(ptr, mathType, context, "mathPtr");
    };
    MathClassReference.prototype.getFields = function () {
        return [];
    };
    MathClassReference.prototype.getConstructor = function () {
        throw new Error("Math object cannot be instantiated");
    };
    MathClassReference.prototype.objectFor = function (pointer, type) {
        return new math_object_reference_1.MathObjectReference(pointer, type, this);
    };
    return MathClassReference;
}(class_reference_1.ClassReference));
exports.MathClassReference = MathClassReference;
//# sourceMappingURL=math-class-reference.js.map