"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var computed_object_property_reference_builder_1 = require("../util/computed-object-property-reference-builder");
var built_in_object_reference_1 = require("./built-in-object-reference");
var primitive_1 = require("./primitive");
var unresolved_method_reference_1 = require("./unresolved-method-reference");
/**
 * Wrapper for the built in Math object
 */
var MathObjectReference = (function (_super) {
    __extends(MathObjectReference, _super);
    function MathObjectReference(pointer, mathType, mathClass) {
        var _this = _super.call(this, pointer, mathType, mathClass) || this;
        _this.typeName = "Math";
        return _this;
    }
    MathObjectReference.prototype.createFunctionFor = function (symbol, signatures, propertyAccessExpression, context) {
        switch (symbol.name) {
            // use the llvm intrinsic whenever a web assembly instruction exists
            case "pow":
            case "sqrt":
            case "log":
            case "sin":
            case "cos":
            case "max":
                return unresolved_method_reference_1.UnresolvedMethodReference.createRuntimeMethod(this, signatures, context, { readnone: true, noUnwind: true });
            default:
                throw code_generation_diagnostic_1.CodeGenerationDiagnostic.builtInMethodNotSupported(propertyAccessExpression, "Math", symbol.name);
        }
    };
    MathObjectReference.prototype.createPropertyReference = function (symbol, propertyAccess, context) {
        switch (symbol.name) {
            case "PI":
                return computed_object_property_reference_builder_1.ComputedObjectPropertyReferenceBuilder
                    .forProperty(propertyAccess, context)
                    .readonly()
                    .fromRuntime()
                    .build(this);
            default:
                return this.throwUnsupportedBuiltIn(propertyAccess);
        }
    };
    /**
     * Calls the pow function
     * @param lhs the left hand side value (base)
     * @param lhsType the type of the lhs value
     * @param rhs the right hand side value (exponent)
     * @param rhsType the type of the rhs value
     * @param numberType the type of the pow result
     * @param context the context
     * @return the result of the pow operation
     */
    MathObjectReference.pow = function (lhs, lhsType, rhs, rhsType, numberType, context) {
        var mathSymbol = context.compilationContext.builtIns.get("Math");
        var mathAllocation = context.scope.getVariable(mathSymbol);
        var mathObject = mathAllocation.dereference(context);
        var powSymbol = mathObject.type.getProperty("pow");
        var powSignature = context.typeChecker.getSignatureFromDeclaration(powSymbol.valueDeclaration);
        var method = unresolved_method_reference_1.UnresolvedMethodReference.createRuntimeMethod(mathObject, [powSignature], context, { readnone: true, noUnwind: true });
        var args = [primitive_1.Primitive.toNumber(lhs, lhsType, numberType, context).generateIR(), primitive_1.Primitive.toNumber(rhs, rhsType, numberType, context).generateIR()];
        return method.invokeWith(args, context);
    };
    return MathObjectReference;
}(built_in_object_reference_1.BuiltInObjectReference));
exports.MathObjectReference = MathObjectReference;
//# sourceMappingURL=math-object-reference.js.map