"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Reference to a specific object index (x[10])
 */
var ObjectIndexReference = (function () {
    function ObjectIndexReference(type, object, index, getter, setter) {
        this.type = type;
        this.object = object;
        this.index = index;
        this.getter = getter;
        this.setter = setter;
    }
    ObjectIndexReference.prototype.generateIR = function (context) {
        return context.builder.createCall(this.getter, [this.object.generateIR(context), this.index.generateIR(context)], "[i]");
    };
    ObjectIndexReference.prototype.getValue = function (context) {
        var call = this.generateIR(context);
        return context.value(call, this.type);
    };
    ObjectIndexReference.prototype.generateAssignmentIR = function (value, context) {
        var valueToSet = value.generateIR(context);
        var objectPtr = this.object.generateIR(context);
        var indexValue = this.index.generateIR(context);
        context.builder.createCall(this.setter, [objectPtr, indexValue, valueToSet]);
    };
    ObjectIndexReference.prototype.isAssignable = function () {
        return true;
    };
    ObjectIndexReference.prototype.isObject = function () {
        return false;
    };
    ObjectIndexReference.prototype.dereference = function (context) {
        return this.getValue(context);
    };
    ObjectIndexReference.prototype.castImplicit = function (type, context) {
        return this.dereference(context).castImplicit(type, context);
    };
    return ObjectIndexReference;
}());
exports.ObjectIndexReference = ObjectIndexReference;
//# sourceMappingURL=object-index-reference.js.map