"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var runtime_system_name_mangler_1 = require("../runtime-system-name-mangler");
var abstract_function_reference_1 = require("./abstract-function-reference");
var function_factory_1 = require("./function-factory");
var resolved_function_1 = require("./resolved-function");
var DEFAULT_RUNTIME_FUNCTION_PROPERTIES = { linkage: llvm.LinkageTypes.ExternalLinkage, alwaysInline: true };
/**
 * Reference to a specific overload of a function.
 */
var ResolvedFunctionReference = (function (_super) {
    __extends(ResolvedFunctionReference, _super);
    function ResolvedFunctionReference(fn, resolvedFunction) {
        var _this = _super.call(this) || this;
        _this.fn = fn;
        _this.resolvedFunction = resolvedFunction;
        return _this;
    }
    /**
     * Creates a new reference to the specified runtime function
     * @param resolvedFunction the resolved overload of the runtime function
     * @param context the context
     * @param functionProperties the function properties
     * @return {ResolvedFunctionReference} the reference to the specified runtime function overload
     */
    ResolvedFunctionReference.createRuntimeFunction = function (resolvedFunction, context, functionProperties) {
        functionProperties = Object.assign({}, DEFAULT_RUNTIME_FUNCTION_PROPERTIES, functionProperties);
        var llvmFunctionFactory = new function_factory_1.FunctionFactory(new runtime_system_name_mangler_1.RuntimeSystemNameMangler(context.compilationContext));
        var fn = llvmFunctionFactory.getOrCreate(resolvedFunction, resolvedFunction.parameters.length, context, functionProperties);
        return new ResolvedFunctionReference(fn, resolvedFunction);
    };
    /**
     * Creates a new reference to the overload defined by the passed in signature
     * @param fn the llvm function that implements the given signature
     * @param signature the signature of the function
     * @param context the context
     * @return {ResolvedFunctionReference} a reference to the given llvm function with the given signature
     */
    ResolvedFunctionReference.createForSignature = function (fn, signature, context) {
        var resolvedFunction = resolved_function_1.createResolvedFunctionFromSignature(signature, context.compilationContext);
        return new ResolvedFunctionReference(fn, resolvedFunction);
    };
    ResolvedFunctionReference.create = function (fn, resolvedFunction) {
        return new ResolvedFunctionReference(fn, resolvedFunction);
    };
    ResolvedFunctionReference.prototype.getResolvedFunction = function () {
        return this.resolvedFunction;
    };
    ResolvedFunctionReference.prototype.getLLVMFunction = function () {
        return this.fn;
    };
    return ResolvedFunctionReference;
}(abstract_function_reference_1.AbstractFunctionReference));
exports.ResolvedFunctionReference = ResolvedFunctionReference;
//# sourceMappingURL=resolved-function-reference.js.map