"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var class_reference_1 = require("./class-reference");
var speedyjs_constructor_function_reference_1 = require("./speedyjs-constructor-function-reference");
var speedyjs_object_reference_1 = require("./speedyjs-object-reference");
var SpeedyJSClassReference = (function (_super) {
    __extends(SpeedyJSClassReference, _super);
    function SpeedyJSClassReference(typeInformation, objectType, compilationContext) {
        var _this = _super.call(this, typeInformation, objectType.getSymbol(), compilationContext) || this;
        _this.objectType = objectType;
        return _this;
    }
    SpeedyJSClassReference.create = function (type, context) {
        var baseTypes = type.getBaseTypes();
        var declaration = type.getSymbol().valueDeclaration;
        if (baseTypes && type.getBaseTypes().length > 0) {
            throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedClassInheritance(declaration);
        }
        if (type.objectFlags & ts.ObjectFlags.Reference) {
            var typeReference = type;
            if (typeReference.typeArguments && typeReference.typeArguments.length > 0) {
                throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedGenericClass(declaration);
            }
        }
        var typeInformation = class_reference_1.ClassReference.createTypeDescriptor(type.getSymbol(), context);
        return new SpeedyJSClassReference(typeInformation, type, context.compilationContext);
    };
    Object.defineProperty(SpeedyJSClassReference.prototype, "type", {
        get: function () {
            return this.objectType;
        },
        enumerable: true,
        configurable: true
    });
    SpeedyJSClassReference.prototype.getFields = function (type, context) {
        var fields = this.type.getApparentProperties().filter(function (property) { return property.flags & ts.SymbolFlags.Property; });
        return fields.map(function (field) {
            return {
                name: field.getName(),
                type: context.typeChecker.getTypeOfSymbolAtLocation(field, field.valueDeclaration)
            };
        });
    };
    SpeedyJSClassReference.prototype.getConstructor = function (newExpression, context) {
        var signature = context.typeChecker.getResolvedSignature(newExpression);
        context.requiresGc = true;
        return speedyjs_constructor_function_reference_1.SpeedyJSConstructorFunctionReference.create(signature, this, context);
    };
    SpeedyJSClassReference.prototype.objectFor = function (address, type, context) {
        assert(type.flags & ts.TypeFlags.Object, "Requires an object type");
        return new speedyjs_object_reference_1.SpeedyJSObjectReference(address, type, this);
    };
    return SpeedyJSClassReference;
}(class_reference_1.ClassReference));
exports.SpeedyJSClassReference = SpeedyJSClassReference;
//# sourceMappingURL=speedy-js-class-reference.js.map