"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var llvm = require("llvm-node");
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var default_name_mangler_1 = require("../default-name-mangler");
var function_definition_builder_1 = require("../util/function-definition-builder");
var function_factory_1 = require("./function-factory");
function verifyIsSupportedSpeedyJSFunction(declaration, context) {
    // tslint:disable-next-line: max-line-length
    if (!(declaration.kind === ts.SyntaxKind.FunctionDeclaration || declaration.kind === ts.SyntaxKind.MethodDeclaration || declaration.kind === ts.SyntaxKind.Constructor)) {
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedFunctionDeclaration(declaration);
    }
    var functionDeclaration = declaration;
    if (functionDeclaration.typeParameters && functionDeclaration.typeParameters.length > 0) {
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedGenericFunction(functionDeclaration);
    }
    // tslint:disable-next-line: max-line-length
    if (functionDeclaration.kind === ts.SyntaxKind.FunctionDeclaration && functionDeclaration.parent && functionDeclaration.parent.kind !== ts.SyntaxKind.SourceFile) {
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedNestedFunctionDeclaration(functionDeclaration);
    }
    if (context.typeChecker.isImplementationOfOverload(functionDeclaration)) {
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedOverloadedFunctionDeclaration(functionDeclaration);
    }
}
exports.verifyIsSupportedSpeedyJSFunction = verifyIsSupportedSpeedyJSFunction;
/**
 * Function factory for functions marked with "speedyjs"
 */
var SpeedyJSFunctionFactory = (function (_super) {
    __extends(SpeedyJSFunctionFactory, _super);
    function SpeedyJSFunctionFactory(compilationContext) {
        return _super.call(this, new default_name_mangler_1.DefaultNameMangler(compilationContext)) || this;
    }
    SpeedyJSFunctionFactory.prototype.mangleFunctionName = function (resolvedFunction, typesOfUsedParameters) {
        if (resolvedFunction.async) {
            return resolvedFunction.functionName;
        }
        return _super.prototype.mangleFunctionName.call(this, resolvedFunction, typesOfUsedParameters);
    };
    SpeedyJSFunctionFactory.prototype.getDefaultFunctionProperties = function () {
        return Object.assign({}, _super.prototype.getDefaultFunctionProperties.call(this), {
            visibility: llvm.VisibilityTypes.Hidden
        });
    };
    SpeedyJSFunctionFactory.prototype.createFunction = function (mangledName, resolvedFunction, numberOfArguments, context, properties, objectReference) {
        var definition = resolvedFunction.definition;
        assert(definition, "Functions only with a declaration cannot be defined");
        verifyIsSupportedSpeedyJSFunction(definition, context);
        assert(definition.body, "Cannot define a function without a body");
        var fn = _super.prototype.createFunction.call(this, mangledName, resolvedFunction, numberOfArguments, context, properties, objectReference);
        var childContext = context.createChildContext();
        function_definition_builder_1.FunctionDefinitionBuilder.create(fn, resolvedFunction, childContext).define();
        return fn;
    };
    return SpeedyJSFunctionFactory;
}(function_factory_1.FunctionFactory));
exports.SpeedyJSFunctionFactory = SpeedyJSFunctionFactory;
//# sourceMappingURL=speedyjs-function-factory.js.map