"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var code_generation_diagnostic_1 = require("../../code-generation-diagnostic");
var types_1 = require("../util/types");
var address_lvalue_1 = require("./address-lvalue");
var object_property_reference_1 = require("./object-property-reference");
var unresolved_method_reference_1 = require("./unresolved-method-reference");
var SpeedyJSObjectReference = (function () {
    function SpeedyJSObjectReference(address, type, clazz) {
        this.address = address;
        this.type = type;
        this.clazz = clazz;
    }
    SpeedyJSObjectReference.prototype.getTypeStoreSize = function (context) {
        return this.clazz.getTypeStoreSize(this.type, context);
    };
    SpeedyJSObjectReference.prototype.getProperty = function (property, context) {
        var symbol = context.typeChecker.getSymbolAtLocation(property);
        if (symbol.flags & ts.SymbolFlags.Property) {
            var type_1 = context.typeChecker.getTypeAtLocation(property);
            return object_property_reference_1.ObjectPropertyReference.createFieldProperty(type_1, this, symbol);
        }
        // otherwise it is a method
        var type = context.typeChecker.getTypeAtLocation(property);
        var apparentType = context.typeChecker.getApparentType(type);
        var signatures = context.typeChecker.getSignaturesOfType(apparentType, ts.SignatureKind.Call);
        return unresolved_method_reference_1.UnresolvedMethodReference.createMethod(this, signatures, context);
    };
    SpeedyJSObjectReference.prototype.getIndexer = function (element, context) {
        throw code_generation_diagnostic_1.CodeGenerationDiagnostic.unsupportedIndexer(element);
    };
    SpeedyJSObjectReference.prototype.isAssignable = function () {
        return false;
    };
    SpeedyJSObjectReference.prototype.isObject = function () {
        return true;
    };
    SpeedyJSObjectReference.prototype.castImplicit = function (type, context) {
        if (this.type === type || types_1.isMaybeObjectType(type) && type.types.indexOf(this.type) !== -1) {
            return this;
        }
        // casting it to undefined. Casts to other types are not yet supported
        if (type.flags & ts.TypeFlags.Undefined) {
            var castedPtr = context.builder.createBitCast(this.generateIR(context), types_1.toLLVMType(type, context));
            return this.clazz.objectFor(new address_lvalue_1.AddressLValue(castedPtr, type), this.type, context);
        }
        if (type.flags & ts.TypeFlags.TypeParameter) {
            var typeParameter = type;
            // e.g. function with return type this
            if (typeParameter.constraint === this.type) {
                return this;
            }
        }
        return undefined;
    };
    SpeedyJSObjectReference.prototype.dereference = function (context) {
        return this;
    };
    SpeedyJSObjectReference.prototype.generateIR = function (context) {
        return this.address.get(context);
    };
    return SpeedyJSObjectReference;
}());
exports.SpeedyJSObjectReference = SpeedyJSObjectReference;
//# sourceMappingURL=speedyjs-object-reference.js.map