"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var ts = require("typescript");
var types_1 = require("../util/types");
var primitive_1 = require("./primitive");
/**
 * Represents the undefined value.
 * Is a i8 pointer value by default but can be casted to the target type (supports, boolean, int, number and object)
 */
var Undefined = (function () {
    function Undefined(nullPtr) {
        this.nullPtr = nullPtr;
    }
    Undefined.create = function (context) {
        return new Undefined(llvm.Constant.getNullValue(llvm.Type.getInt8PtrTy(context.llvmContext)));
    };
    Undefined.prototype.isAssignable = function () {
        return false;
    };
    Undefined.prototype.isObject = function () {
        return false;
    };
    Undefined.prototype.dereference = function (context) {
        return this;
    };
    Undefined.prototype.generateIR = function (context) {
        return this.nullPtr;
    };
    Undefined.prototype.castImplicit = function (type, context) {
        if (type.flags & ts.TypeFlags.BooleanLike) {
            return new primitive_1.Primitive(llvm.ConstantInt.getFalse(context.llvmContext), type);
        }
        if (type.flags & ts.TypeFlags.IntLike) {
            return new primitive_1.Primitive(llvm.ConstantInt.get(context.llvmContext, 0), type);
        }
        if (type.flags & ts.TypeFlags.NumberLike) {
            return new primitive_1.Primitive(llvm.ConstantFP.getNaN(llvm.Type.getDoubleTy(context.llvmContext)), type);
        }
        // cast pointer
        if (type.flags & ts.TypeFlags.Object || types_1.isMaybeObjectType(type) || type.flags & ts.TypeFlags.Undefined) {
            return new Undefined(context.builder.createBitCast(this.generateIR(context), types_1.toLLVMType(type, context)));
        }
        return undefined;
    };
    return Undefined;
}());
exports.Undefined = Undefined;
//# sourceMappingURL=undefined.js.map