"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var llvm = require("llvm-node");
var runtime_system_name_mangler_1 = require("../runtime-system-name-mangler");
var abstract_function_reference_1 = require("./abstract-function-reference");
var function_factory_1 = require("./function-factory");
var speedyjs_function_factory_1 = require("./speedyjs-function-factory");
/**
 * Reference to a probably overloaded function.
 * Implements the logic to resolve the specific overload for a call expression.
 */
var UnresolvedFunctionReference = (function (_super) {
    __extends(UnresolvedFunctionReference, _super);
    function UnresolvedFunctionReference(signatures, llvmFunctionFactory, classType, properties) {
        var _this = _super.call(this, classType) || this;
        _this.signatures = signatures;
        _this.llvmFunctionFactory = llvmFunctionFactory;
        assert(signatures.length, "Cannot reference a function without a signature");
        _this.properties = properties || { linkage: llvm.LinkageTypes.LinkOnceODRLinkage };
        return _this;
    }
    /**
     * Creates a reference to a function reference that has the specified overloads
     * @param signatures the references of the function
     * @param context the context
     * @param classType the class type if the function is an instance or static method of an object
     * @param properties properties for the declared function
     * @return {UnresolvedFunctionReference} the function reference
     */
    UnresolvedFunctionReference.createRuntimeFunction = function (signatures, context, classType, properties) {
        properties = Object.assign({ linkage: llvm.LinkageTypes.ExternalLinkage, alwaysInline: true }, properties);
        var functionFactory = new function_factory_1.FunctionFactory(new runtime_system_name_mangler_1.RuntimeSystemNameMangler(context.compilationContext));
        return new UnresolvedFunctionReference(signatures, functionFactory, classType, properties);
    };
    /**
     * Creates a reference to a function that has the specified overloads
     * @param signatures the signatures of the function
     * @param context the context
     * @param classType the class type if the function is an instance or static method of an object
     * @return {UnresolvedFunctionReference} the function reference
     */
    UnresolvedFunctionReference.createFunction = function (signatures, context, classType) {
        return new UnresolvedFunctionReference(signatures, new speedyjs_function_factory_1.SpeedyJSFunctionFactory(context.compilationContext), classType);
    };
    UnresolvedFunctionReference.prototype.getResolvedFunction = function (context) {
        if (this.signatures.length > 1) {
            throw new Error("Cannot dereference an overloaded function");
        }
        var signature = this.signatures[0];
        if (signature.typeParameters && signature.typeParameters.length) {
            throw new Error("Cannot dereference a generic function");
        }
        if (!signature.declaration.parameters.every(function (parameter) { return !parameter.questionToken; })) {
            throw new Error("Cannot deference a function with optional arguments");
        }
        return this.getResolvedFunctionFromSignature(signature, context.compilationContext);
    };
    UnresolvedFunctionReference.prototype.getLLVMFunction = function (resolvedFunction, context, passedArguments) {
        var numberOfArguments = passedArguments ? passedArguments.length : resolvedFunction.parameters.length;
        return this.llvmFunctionFactory.getOrCreate(resolvedFunction, numberOfArguments, context, this.properties);
    };
    return UnresolvedFunctionReference;
}(abstract_function_reference_1.AbstractFunctionReference));
exports.UnresolvedFunctionReference = UnresolvedFunctionReference;
//# sourceMappingURL=unresolved-function-reference.js.map