"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var llvm = require("llvm-node");
var runtime_system_name_mangler_1 = require("../runtime-system-name-mangler");
var function_factory_1 = require("./function-factory");
var speedyjs_function_factory_1 = require("./speedyjs-function-factory");
var unresolved_function_reference_1 = require("./unresolved-function-reference");
/**
 * Reference to a possibly overloaded instance method
 */
var UnresolvedMethodReference = (function (_super) {
    __extends(UnresolvedMethodReference, _super);
    function UnresolvedMethodReference(object, signatures, llvmFunctionFactory, properties) {
        var _this = _super.call(this, signatures, llvmFunctionFactory, object.type, properties) || this;
        _this.object = object;
        return _this;
    }
    /**
     * Creates a reference to an instance method in the runtime
     * @param object the object
     * @param signatures the signatures of the method
     * @param context the context
     * @param properties the function properties
     * @return {UnresolvedMethodReference} the reference to this method
     */
    UnresolvedMethodReference.createRuntimeMethod = function (object, signatures, context, properties) {
        var functionFactory = new function_factory_1.FunctionFactory(new runtime_system_name_mangler_1.RuntimeSystemNameMangler(context.compilationContext));
        properties = Object.assign({ linkage: llvm.LinkageTypes.ExternalLinkage, alwaysInline: true }, properties);
        return new UnresolvedMethodReference(object, signatures, functionFactory, properties);
    };
    /**
     * Creates a reference to a method that has the specified overloads
     * @param object the object to which the method belongs
     * @param signatures the signatures of the method
     * @param context the context
     * @return the reference to the method
     */
    UnresolvedMethodReference.createMethod = function (object, signatures, context) {
        return new UnresolvedMethodReference(object, signatures, new speedyjs_function_factory_1.SpeedyJSFunctionFactory(context.compilationContext));
    };
    UnresolvedMethodReference.prototype.getLLVMFunction = function (resolvedFunction, context, passedArguments) {
        var numberOfArguments = passedArguments ? passedArguments.length : resolvedFunction.parameters.length;
        return this.llvmFunctionFactory.getOrCreateInstanceMethod(this.object, resolvedFunction, numberOfArguments, context, this.properties);
    };
    UnresolvedMethodReference.prototype.getCallArguments = function (resolvedFunction, passedArguments, callerContext) {
        return [this.object.generateIR(callerContext)].concat(_super.prototype.getCallArguments.call(this, resolvedFunction, passedArguments, callerContext));
    };
    return UnresolvedMethodReference;
}(unresolved_function_reference_1.UnresolvedFunctionReference));
exports.UnresolvedMethodReference = UnresolvedMethodReference;
//# sourceMappingURL=unresolved-method-reference.js.map