"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tools_1 = require("./tools");
/**
 * Identifies the symbols exported by an LLVM .bc File
 */
var LLVMByteCodeSymbolsResolver = (function () {
    function LLVMByteCodeSymbolsResolver() {
        this.cache = new Map();
    }
    /**
     * Returns the symbols exported by the given file
     * @param objectFile the path to the object file
     * @return the symbols of this object files
     */
    LLVMByteCodeSymbolsResolver.prototype.getSymbols = function (objectFile) {
        var result;
        var cached = this.cache.get(objectFile);
        if (cached) {
            result = cached;
        }
        else {
            result = this.parseSymbolsFromObjectFile(objectFile);
            this.cache.set(objectFile, result);
        }
        return Object.create(result);
    };
    LLVMByteCodeSymbolsResolver.prototype.parseSymbolsFromObjectFile = function (objectFile) {
        var output = tools_1.execLLVM("llvm-nm", "\"" + objectFile + "\"");
        var symbols = { undefined: new Set(), common: new Set(), defined: new Set() };
        for (var _i = 0, _a = output.split("\n"); _i < _a.length; _i++) {
            var line = _a[_i];
            if (line.length === 0 || line.indexOf(":") > -1) {
                continue;
            }
            var parts = line.split(" ").filter(function (part) { return part.length > 0; });
            if (parts.length === 3 && parts[0] === "--------") {
                parts.shift();
            }
            if (parts.length === 2) {
                var typeCode = parts[0], symbol = parts[1];
                if (typeCode === "U") {
                    symbols.undefined.add(symbol);
                }
                else if (typeCode === "C") {
                    symbols.common.add(symbol);
                }
                else if (typeCode === "W" || typeCode === "t" || typeCode === "T" || typeCode === "d" || typeCode === "D") {
                    symbols.defined.add(symbol);
                }
            }
        }
        return symbols;
    };
    return LLVMByteCodeSymbolsResolver;
}());
exports.LLVMByteCodeSymbolsResolver = LLVMByteCodeSymbolsResolver;
//# sourceMappingURL=llvm-nm.js.map