"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var debug = require("debug");
var tools_1 = require("./tools");
var LOG = debug("external-tools/llvm-opt");
var EXECUTABLE_NAME = "opt";
// tslint:disable-next-line:max-line-length
var LINK_TIME_OPTIMIZATIONS = "-strip-debug -internalize -globaldce -disable-loop-vectorization -disable-slp-vectorization -vectorize-loops=false -vectorize-slp=false -vectorize-slp-aggressive=false"; // Vectorization is not yet supported by the linker backend llc
var DEFAULT_PUBLIC = "speedyJsGc,malloc,free,memcpy,memset,memmove,__errno_location,__cxa_can_catch,__cxa_is_pointer_type";
/**
 * Executes the LLVM Optimizer on the given input file
 * @param filename the input file (absolute path)
 * @param optimizedFileName the name of the target file
 * @param level the optimization level
 * @returns the name of the optimized file
 */
function optimize(filename, optimizedFileName, level) {
    LOG("Optimization of file " + filename);
    LOG(tools_1.execLLVM(EXECUTABLE_NAME, "\"" + filename + "\" -o \"" + optimizedFileName + "\" -O" + level + " -loop-unswitch -loop-unswitch -licm -irce"));
    return optimizedFileName;
}
exports.optimize = optimize;
/**
 * Executes the link time optimizations on the given input file
 * @param filename the input file (absolute path)
 * @param publicFunctions name of the public functions
 * @param optimizedFileName the name of the target file
 * @param level the used optimization level. If the level is at least 2, than link time optimizations are performed.
 * @returns the name of the optimized file
 */
function optimizeLinked(filename, publicFunctions, optimizedFileName, level) {
    var publicApi = publicFunctions.concat(DEFAULT_PUBLIC).join(",");
    var linkTimeOpts = ["2", "3", "z", "s"].indexOf(level) !== -1;
    var optimizations = (linkTimeOpts ? "-std-link-opts" : "") + " " + LINK_TIME_OPTIMIZATIONS;
    var args = "\"" + filename + "\" -o \"" + optimizedFileName + "\" " + optimizations + " -internalize-public-api-list=\"" + publicApi + "\"";
    LOG("Link time optimization of file " + filename);
    LOG(tools_1.execLLVM(EXECUTABLE_NAME, args));
    return optimizedFileName;
}
exports.optimizeLinked = optimizeLinked;
//# sourceMappingURL=llvm-opt.js.map