"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var child_process = require("child_process");
var debug = require("debug");
var path = require("path");
var ts = require("typescript");
var CONFIGURATION_FILE = path.normalize(path.join(__dirname, "../../tools/configuration.json"));
var log = debug("external-tools/tools");
/**
 * Returns the path where the llvm executables are located
 */
function getConfiguration() {
    if (!ts.sys.fileExists(CONFIGURATION_FILE)) {
        throw new Error("The configuration file '" + CONFIGURATION_FILE + " for the installations does not exist. Install the package again");
    }
    var content = ts.sys.readFile(CONFIGURATION_FILE);
    return JSON.parse(content);
}
exports.getConfiguration = getConfiguration;
/**
 * executes a llvm tool
 * @param tool the name of the binary
 * @param args the arguments to pass
 * @param cwd optionally, the working directory where the command is to be executed
 * @return {string} the result of executing this command
 */
function execLLVM(tool, args, cwd) {
    var toolPath = path.join(getConfiguration().LLVM, tool);
    if (!ts.sys.fileExists(toolPath)) {
        throw new Error("LLVM executable " + toolPath + " is missing");
    }
    return execute(toolPath, args, cwd);
}
exports.execLLVM = execLLVM;
/**
 * Executes a binaryen command
 * @param tool the name of the binaryen executable
 * @param args the arguments to pass
 * @return {string} the output of the execution
 */
function execBinaryen(tool, args) {
    var binaryenPath = path.resolve(__dirname + "/../../", getConfiguration().BINARYEN);
    var toolPath = path.join(binaryenPath, "bin", tool);
    if (!ts.sys.fileExists(toolPath)) {
        throw new Error("BINARYEN executable " + toolPath + " is missing");
    }
    return execute(toolPath, args);
}
exports.execBinaryen = execBinaryen;
function execute(tool, args, cwd) {
    var env = Object.create(process.env);
    var command = tool + " " + args;
    log("Execute command '" + command + "'");
    var output = child_process.execSync(tool + " " + args, { env: env, cwd: cwd });
    return outputToString(output);
}
function outputToString(output) {
    if (typeof (output) === "string") {
        return output;
    }
    return output.toString("utf-8");
}
//# sourceMappingURL=tools.js.map