"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var default_wasm_file_writer_1 = require("./default-wasm-file-writer");
/**
 * Initializes the compiler options with the default values
 * @param compilerOptions the compiler options to initialize
 * @return {SpeedyJSCompilerOptions} the initialized compiler options
 */
function initializeCompilerOptions(compilerOptions) {
    var defaults = {
        unsafe: true,
        emitLLVM: false,
        saveWast: false,
        binaryenOpt: true,
        initialMemory: 16 * 1024 * 1024,
        totalStack: 5 * 1024 * 104,
        globalBase: 8,
        saveBc: false,
        disableHeapNukeOnExit: false,
        exposeGc: false,
        exportGc: false,
        optimizationLevel: "2",
        wasmFileWriter: new default_wasm_file_writer_1.DefaultWasmFileWriter()
    };
    for (var _i = 0, _a = Object.keys(defaults); _i < _a.length; _i++) {
        var key = _a[_i];
        compilerOptions[key] = typeof (compilerOptions[key]) === "undefined" ? defaults[key] : compilerOptions[key];
    }
    compilerOptions.strictNullChecks = true;
    compilerOptions.noImplicitAny = true; // speedy js cannot handle any
    compilerOptions.noImplicitReturns = true;
    compilerOptions.lib = compilerOptions.lib || ["lib.es2015.d.ts"];
    return compilerOptions;
}
exports.initializeCompilerOptions = initializeCompilerOptions;
//# sourceMappingURL=speedyjs-compiler-options.js.map