"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
/**
 * tests if the passed function is a speedy js function
 * @param fun the function to test
 * @return {boolean} true if the function is a speedy js function
 */
function isSpeedyJSFunction(fun) {
    if (!fun.body || !isBlock(fun.body)) {
        return false;
    }
    for (var _i = 0, _a = fun.body.statements; _i < _a.length; _i++) {
        var statement = _a[_i];
        if (isPrologueDirective(statement)) {
            if (statement.expression.text === "use speedyjs") {
                return true;
            }
        }
        else {
            break;
        }
    }
    return false;
}
exports.isSpeedyJSFunction = isSpeedyJSFunction;
/**
 * A speedy js function has the async modifier and contains "use speedyjs" directive
 * @param fun the function to test
 * @return {boolean} true if it is a speedy js entry function
 */
function isSpeedyJSEntryFunction(fun) {
    return isSpeedyJSFunction(fun) &&
        fun.kind === ts.SyntaxKind.FunctionDeclaration &&
        !!fun.modifiers &&
        !!fun.modifiers.find(function (modifier) { return modifier.kind === ts.SyntaxKind.AsyncKeyword; });
}
exports.isSpeedyJSEntryFunction = isSpeedyJSEntryFunction;
function isPrologueDirective(node) {
    return node.kind === ts.SyntaxKind.ExpressionStatement && node.expression.kind === ts.SyntaxKind.StringLiteral;
}
function isBlock(node) {
    return node.kind === ts.SyntaxKind.Block;
}
//# sourceMappingURL=speedyjs-function.js.map